@@ -70,7 +70,7 @@ int		htmlDocDump		(FILE *f,
 					 xmlDocPtr cur);
 int		htmlSaveFile		(const char *filename,
 					 xmlDocPtr cur);
-void		htmlNodeDump		(xmlBufferPtr buf,
+int		htmlNodeDump		(xmlBufferPtr buf,
 					 xmlDocPtr doc,
 					 xmlNodePtr cur);
 void		htmlNodeDumpFile	(FILE *out,
@@ -103,6 +103,9 @@ void		htmlDocContentDumpFormatOutput(xmlOutputBufferPtr buf,
 					 int format);
 
 int		htmlIsBooleanAttr	(const xmlChar *name);
+void htmlNodeDumpOutput(xmlOutputBufferPtr buf, xmlDocPtr doc,
+	                xmlNodePtr cur, const char *encoding);
+
 
 
 
