@@ -11,6 +11,9 @@
 
 #include <libxml/tree.h>
 #include <libxml/xmlIO.h>
+#ifdef LIBXML_SCHEMAS_ENABLED
+#include <libxml/relaxng.h>
+#endif
 
 #ifdef __cplusplus
 extern "C" {
@@ -106,6 +109,15 @@ int		xmlTextReaderGetParserProp	(xmlTextReaderPtr reader,
 						 int prop);
 xmlNodePtr	xmlTextReaderCurrentNode	(xmlTextReaderPtr reader);
 xmlDocPtr	xmlTextReaderCurrentDoc		(xmlTextReaderPtr reader);
+xmlNodePtr	xmlTextReaderExpand		(xmlTextReaderPtr reader);
+int		xmlTextReaderNext		(xmlTextReaderPtr reader);
+int		xmlTextReaderIsValid		(xmlTextReaderPtr reader);
+#ifdef LIBXML_SCHEMAS_ENABLED
+int		xmlTextReaderRelaxNGValidate	(xmlTextReaderPtr reader,
+						 const char *rng);
+int		xmlTextReaderRelaxNGSetSchema	(xmlTextReaderPtr reader,
+						 xmlRelaxNGPtr schema);
+#endif
 
 /*
  * Error handling extensions
