@@ -1,6 +1,6 @@
 /*************************************************************************
  *
- * $Id: triodef.h,v 1.5 2002/10/22 17:35:33 veillard Exp $
+ * $Id: triodef.h,v 1.6 2003/04/03 15:28:28 veillard Exp $
  *
  * Copyright (C) 2001 Bjorn Reese <breese@users.sourceforge.net>
  *
@@ -42,11 +42,17 @@
 # define TRIO_COMPILER_BCB
 #endif
 
-#if defined(unix) || defined(__unix) || defined(__unix__)
+#if defined(VMS) || defined(__VMS)
+/*
+ * VMS is placed first to avoid identifying the platform as Unix
+ * based on the DECC compiler later on.
+ */
+# define TRIO_PLATFORM_VMS
+#elif defined(unix) || defined(__unix) || defined(__unix__)
 # define TRIO_PLATFORM_UNIX
 #elif defined(TRIO_COMPILER_XLC) || defined(_AIX)
 # define TRIO_PLATFORM_UNIX
-#elif ( defined(TRIO_COMPILER_DECC) && !defined(__VMS) ) || defined(__osf__)
+#elif defined(TRIO_COMPILER_DECC) || defined(__osf___)
 # define TRIO_PLATFORM_UNIX
 #elif defined(__NetBSD__)
 # define TRIO_PLATFORM_UNIX
@@ -59,13 +65,23 @@
 # define TRIO_PLATFORM_UNIX
 #elif defined(TRIO_COMPILER_MSVC) || defined(WIN32) || defined(_WIN32)
 # define TRIO_PLATFORM_WIN32
-#elif defined(VMS) || defined(__VMS)
-# define TRIO_PLATFORM_VMS
 #elif defined(mpeix) || defined(__mpexl)
 # define TRIO_PLATFORM_MPEIX
 #endif
 
-#if defined(__STDC__) || defined(TRIO_COMPILER_MSVC)
+#if defined(_AIX)
+# define TRIO_PLATFORM_AIX
+#elif defined(__hpux)
+# define TRIO_PLATFORM_HPUX
+#elif defined(sun) || defined(__sun__)
+# if defined(__SVR4) || defined(__svr4__)
+#  define TRIO_PLATFORM_SOLARIS
+# else
+#  define TRIO_PLATFORM_SUNOS
+# endif
+#endif
+
+#if defined(__STDC__) || defined(TRIO_COMPILER_MSVC) || defined(TRIO_COMPILER_BCB)
 # define TRIO_COMPILER_SUPPORTS_C89
 # if defined(__STDC_VERSION__)
 #  define TRIO_COMPILER_SUPPORTS_C90
@@ -82,18 +98,6 @@
 # endif
 #endif
 
-#if defined(TRIO_PLATFORM_VMS)
- /* The compiler does support C99 but the library still does not have things
-  * the standard requires (like nan() and strtof()) as of __CRTL_VER 70300022.
-  */
-# undef TRIO_COMPILER_SUPPORTS_C99
-
- /* Computations done with constants at compile time can trigger these
-  * even when compiling with IEEE enabled.
-  */
-#  pragma message disable (UNDERFLOW,FLOATOVERFL)
-#endif /* TRIO_PLATFORM_VMS */
-
 #if defined(_XOPEN_SOURCE)
 # if defined(_XOPEN_SOURCE_EXTENDED)
 #  define TRIO_COMPILER_SUPPORTS_UNIX95
@@ -141,7 +145,7 @@ typedef char * trio_pointer_t;
 # define TRIO_VARGS4(list,a1,a2,a3,a4) list a1; a2; a3; a4
 # define TRIO_VARGS5(list,a1,a2,a3,a4,a5) list a1; a2; a3; a4; a5
 # define TRIO_VA_DECL va_dcl
-# define TRIO_VA_START(x,y) va_start((x))
+# define TRIO_VA_START(x,y) va_start(x)
 # define TRIO_VA_END(x) va_end(x)
 #else /* ANSI C */
 # define TRIO_CONST const
@@ -163,7 +167,7 @@ typedef void * trio_pointer_t;
 # define TRIO_VARGS4 TRIO_ARGS4
 # define TRIO_VARGS5 TRIO_ARGS5
 # define TRIO_VA_DECL ...
-# define TRIO_VA_START(x,y) va_start((x),(y))
+# define TRIO_VA_START(x,y) va_start(x,y)
 # define TRIO_VA_END(x) va_end(x)
 #endif
 
@@ -179,4 +183,38 @@ typedef void * trio_pointer_t;
 # define TRIO_INLINE
 #endif
 
+/*************************************************************************
+ * Workarounds
+ */
+
+#if defined(TRIO_PLATFORM_VMS)
+/*
+ * Computations done with constants at compile time can trigger these
+ * even when compiling with IEEE enabled.
+ */
+# pragma message disable (UNDERFLOW, FLOATOVERFL)
+
+# if (__CRTL_VER > 80000000)
+/*
+ * Although the compiler supports C99 language constructs, the C
+ * run-time library does not contain all C99 functions.
+ *
+ * This was the case for 70300022. Update the 80000000 value when
+ * it has been accurately determined what version of the library
+ * supports C99.
+ */
+#  if defined(TRIO_COMPILER_SUPPORTS_C99)
+#   undef TRIO_COMPILER_SUPPORTS_C99
+#  endif
+# endif
+#endif
+
+/*
+ * Not all preprocessors supports the LL token.
+ */
+#if defined(TRIO_COMPILER_BCB)
+#else
+# define TRIO_COMPILER_SUPPORTS_LL
+#endif
+
 #endif /* TRIO_TRIODEF_H */
