@@ -538,6 +538,11 @@ xmlXIncludeRecurseDoc(xmlXIncludeCtxtPtr ctxt, xmlDocPtr doc,
 	    newctxt->incTab[i]->count--;
 	    newctxt->incTab[i] = NULL;
 	}
+
+	/* urlTab may have been reallocated */
+	ctxt->urlTab = newctxt->urlTab;
+	ctxt->urlMax = newctxt->urlMax;
+
 	newctxt->urlMax = 0;
 	newctxt->urlNr = 0;
 	newctxt->urlTab = NULL;
@@ -1059,7 +1064,8 @@ xmlXIncludeMergeEntity(xmlEntityPtr ent, xmlXIncludeMergeDataPtr data,
 	    if ((ent->SystemID != NULL) && (prev->SystemID != NULL)) {
 		if (!xmlStrEqual(ent->SystemID, prev->SystemID))
 		    goto error;
-	    } else if ((ent->SystemID != NULL) && (prev->SystemID != NULL)) {
+	    } else if ((ent->ExternalID != NULL) &&
+		       (prev->ExternalID != NULL)) {
 		if (!xmlStrEqual(ent->ExternalID, prev->ExternalID))
 		    goto error;
 	    } else if ((ent->content != NULL) && (prev->content != NULL)) {
@@ -1073,6 +1079,16 @@ xmlXIncludeMergeEntity(xmlEntityPtr ent, xmlXIncludeMergeDataPtr data,
     }
     return;
 error:
+    switch (ent->etype) {
+        case XML_INTERNAL_PARAMETER_ENTITY:
+        case XML_EXTERNAL_PARAMETER_ENTITY:
+        case XML_INTERNAL_PREDEFINED_ENTITY:
+        case XML_INTERNAL_GENERAL_ENTITY:
+        case XML_EXTERNAL_GENERAL_PARSED_ENTITY:
+	    return;
+        case XML_EXTERNAL_GENERAL_UNPARSED_ENTITY:
+	    break;
+    }
     xmlGenericError(xmlGenericErrorContext,
 		"XInclude: mismatch in redefinition of entity %s\n", ent->name);
     ctxt->nbErrors++;
